/*
 * Created on 27. 7. 2017 15:46:57
 *
 */
package com.jh.core.security.model;

import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;


/**
 * The Class AppUser.
 *
 * @author Jan Hrabal
 */
@Entity
@Table(name = "APP_USER")
@Inheritance(strategy = InheritanceType.JOINED)
public class AppUser {
	
	private static final long serialVersionUID = 1L;

	/** The id. */
	@Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "idGenerator")
    @SequenceGenerator(name = "idGenerator", sequenceName = "ID_GENERATOR", allocationSize = 1)
    @Column(name = "ID", updatable = false, nullable = false)
	private Long id;
	
    /** The version. */
    @Version
    @Column(name = "VERSION")
    private Long version;
    
    /** The deleted. */
    @Column(name = "DELETED")
    private Boolean deleted;
    
    /** The login. */
    @Column(name = "LOGIN")
    private String login;
    
    /** The login. */
    @Column(name = "EMAIL")
    private String email;
    
    /** The password. */
    @Column(name = "PASSWORD")
    private String password; 
    
    /** The password salt. */
    @Column(name = "PASSWORD_SALT")
    private String passwordSalt; 
    
    /** The first name. */
    @Column(name = "FIRST_NAME")
    private String firstName;
    
    /** The last name. */
    @Column(name = "LAST_NAME")
    private String lastName; 
    
    @ManyToMany
    @JoinTable(name="APP_USER_ROLE",
        joinColumns=
            @JoinColumn(name="USER_ID", referencedColumnName="ID"),
        inverseJoinColumns=
            @JoinColumn(name="ROLE_ID", referencedColumnName="ID")
        )
    @JsonIgnore
    private Set<AppRole> roles;
    
    @Column(name = "LOCALE")
    private String locale;
    
    @Column(name = "TIMEZONE_ID")
    private String timezone;
    
    @Column(name = "ACCOUNT_ENABLED")
	private boolean accountEnabled;
	
    @Column(name = "ACCOUNT_EXPIRED")
	private boolean accountExpired;
	
    @Column(name = "ACCOUNT_LOCKED")
	private boolean accountLocked;
	
    @Column(name = "PASSWORD_EXPIRED")
	private boolean passwordExpired;    
    
    @Column(name = "PASSWORD_CHANGED")
	private Date passwordChanged;
	
    @Transient
    @JsonProperty("roles")
    private Map<String, AppRole> rolesMap;
    
    
    public void initRolesMap() {
    	if (roles == null) {
    		rolesMap = Collections.emptyMap();
    		return;
    	}
    	rolesMap = roles.stream().collect(Collectors.toMap(AppRole::getName, r -> r));
    }
    
    /**
     * Gets the email.
     *
     * @return the email
     */
    public String getLogin() {
        return login;
    }

    /**
     * Sets the email.
     *
     * @param email the new email
     */
    public void setLogin(String email) {
        this.login = email;
    }

    /**
     * Gets the version.
     *
     * @return the version
     */
    public Long getVersion() {
        return version;
    }

    /**
     * Sets the version.
     *
     * @param version the new version
     */
    public void setVersion(Long version) {
        this.version = version;
    }

    /**
     * Gets the deleted.
     *
     * @return the deleted
     */
    public Boolean getDeleted() {
        return deleted;
    }

    /**
     * Sets the deleted.
     *
     * @param deleted the new deleted
     */
    public void setDeleted(Boolean deleted) {
        this.deleted = deleted;
    }

    /**
     * Gets the password.
     *
     * @return the password
     */
    public String getPassword() {
        return password;
    }

    /**
     * Sets the password.
     *
     * @param password the new password
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * Gets the password salt.
     *
     * @return the password salt
     */
    public String getPasswordSalt() {
        return passwordSalt;
    }

    /**
     * Sets the password salt.
     *
     * @param passwordSalt the new password salt
     */
    public void setPasswordSalt(String passwordSalt) {
        this.passwordSalt = passwordSalt;
    }

    /**
     * Gets the first name.
     *
     * @return the first name
     */
    public String getFirstName() {
        return firstName;
    }

    /**
     * Sets the first name.
     *
     * @param firstName the new first name
     */
    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    /**
     * Gets the last name.
     *
     * @return the last name
     */
    public String getLastName() {
        return lastName;
    }

    /**
     * Sets the last name.
     *
     * @param lastName the new last name
     */
    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

	public Set<AppRole> getRoles() {
		return roles;
	}

	public void setRoles(Set<AppRole> roles) {
		this.roles = roles;
	}

	public String getLocale() {
		return locale;
	}

	public void setLocale(String locale) {
		this.locale = locale;
	}

	public String getTimezone() {
		return timezone;
	}

	public void setTimezone(String timezone) {
		this.timezone = timezone;
	}

	public boolean isAccountEnabled() {
		return accountEnabled;
	}

	public void setAccountEnabled(boolean accountEnabled) {
		this.accountEnabled = accountEnabled;
	}

	public boolean isAccountExpired() {
		return accountExpired;
	}

	public void setAccountExpired(boolean accountExpired) {
		this.accountExpired = accountExpired;
	}

	public boolean isAccountLocked() {
		return accountLocked;
	}

	public void setAccountLocked(boolean accountLocked) {
		this.accountLocked = accountLocked;
	}

	public boolean isPasswordExpired() {
		return passwordExpired;
	}

	public void setPasswordExpired(boolean passwordExpired) {
		this.passwordExpired = passwordExpired;
	}

	public Date getPasswordChanged() {
		return passwordChanged;
	}

	public void setPasswordChanged(Date passwordChanged) {
		this.passwordChanged = passwordChanged;
	}

	public Map<String, AppRole> getRolesMap() {
		return rolesMap;
	}

	public void setRolesMap(Map<String, AppRole> rolesMap) {
		this.rolesMap = rolesMap;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	@Override
	public int hashCode() {
		if (id == null) {
			return super.hashCode();
		}
		
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	/**
	 * @param obj
	 * @return
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (id == null) {
			return super.equals(obj);
		}
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AppUser other = (AppUser) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
}
